@@ -25,11 +25,11 @@ use arrow::datatypes::UInt32Type;
 use arrow::record_batch::RecordBatch;
 use sqlparser::ast::Ident;
 use sqlparser::dialect::GenericDialect;
-use sqlparser::parser::{Parser, ParserError};
-use sqlparser::tokenizer::{Token, TokenWithLocation};
+use sqlparser::parser::Parser;
 use std::borrow::{Borrow, Cow};
 use std::cmp::Ordering;
 use std::ops::Range;
+use std::sync::Arc;
 
 /// Given column vectors, returns row at `idx`.
 pub fn get_row_at_idx(columns: &[ArrayRef], idx: usize) -> Result<Vec<ScalarValue>> {
@@ -220,56 +220,10 @@ fn needs_quotes(s: &str) -> bool {
     !chars.all(|c| c.is_ascii_lowercase() || c.is_ascii_digit() || c == '_')
 }
 
-// TODO: remove when can use https://github.com/sqlparser-rs/sqlparser-rs/issues/805
 pub(crate) fn parse_identifiers(s: &str) -> Result<Vec<Ident>> {
     let dialect = GenericDialect;
     let mut parser = Parser::new(&dialect).try_with_sql(s)?;
-    let mut idents = vec![];
-
-    // expecting at least one word for identifier
-    match parser.next_token_no_skip() {
-        Some(TokenWithLocation {
-            token: Token::Word(w),
-            ..
-        }) => idents.push(w.to_ident()),
-        Some(TokenWithLocation { token, .. }) => {
-            return Err(ParserError::ParserError(format!(
-                "Unexpected token in identifier: {token}"
-            )))?
-        }
-        None => {
-            return Err(ParserError::ParserError(
-                "Empty input when parsing identifier".to_string(),
-            ))?
-        }
-    };
-
-    while let Some(TokenWithLocation { token, .. }) = parser.next_token_no_skip() {
-        match token {
-            // ensure that optional period is succeeded by another identifier
-            Token::Period => match parser.next_token_no_skip() {
-                Some(TokenWithLocation {
-                    token: Token::Word(w),
-                    ..
-                }) => idents.push(w.to_ident()),
-                Some(TokenWithLocation { token, .. }) => {
-                    return Err(ParserError::ParserError(format!(
-                        "Unexpected token following period in identifier: {token}"
-                    )))?
-                }
-                None => {
-                    return Err(ParserError::ParserError(
-                        "Trailing period in identifier".to_string(),
-                    ))?
-                }
-            },
-            _ => {
-                return Err(ParserError::ParserError(format!(
-                    "Unexpected token in identifier: {token}"
-                )))?
-            }
-        }
-    }
+    let idents = parser.parse_multipart_identifier()?;
     Ok(idents)
 }
 
@@ -336,6 +290,34 @@ pub fn longest_consecutive_prefix<T: Borrow<usize>>(
     count
 }
 
+/// An extension trait for smart pointers. Provides an interface to get a
+/// raw pointer to the data (with metadata stripped away).
+///
+/// This is useful to see if two smart pointers point to the same allocation.
+pub trait DataPtr {
+    /// Returns a raw pointer to the data, stripping away all metadata.
+    fn data_ptr(this: &Self) -> *const ();
+
+    /// Check if two pointers point to the same data.
+    fn data_ptr_eq(this: &Self, other: &Self) -> bool {
+        // Discard pointer metadata (including the v-table).
+        let this = Self::data_ptr(this);
+        let other = Self::data_ptr(other);
+
+        std::ptr::eq(this, other)
+    }
+}
+
+// Currently, it's brittle to compare `Arc`s of dyn traits with `Arc::ptr_eq`
+// due to this check including v-table equality. It may be possible to use
+// `Arc::ptr_eq` directly if a fix to https://github.com/rust-lang/rust/issues/103763
+// is stabilized.
+impl<T: ?Sized> DataPtr for Arc<T> {
+    fn data_ptr(this: &Self) -> *const () {
+        Arc::as_ptr(this) as *const ()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use arrow::array::Float64Array;
@@ -545,64 +527,6 @@ mod tests {
         Ok(())
     }
 
-    #[test]
-    fn test_parse_identifiers() -> Result<()> {
-        let s = "CATALOG.\"F(o)o. \"\"bar\".table";
-        let actual = parse_identifiers(s)?;
-        let expected = vec![
-            Ident {
-                value: "CATALOG".to_string(),
-                quote_style: None,
-            },
-            Ident {
-                value: "F(o)o. \"bar".to_string(),
-                quote_style: Some('"'),
-            },
-            Ident {
-                value: "table".to_string(),
-                quote_style: None,
-            },
-        ];
-        assert_eq!(expected, actual);
-
-        let s = "";
-        let err = parse_identifiers(s).expect_err("didn't fail to parse");
-        assert_eq!(
-            "SQL(ParserError(\"Empty input when parsing identifier\"))",
-            format!("{err:?}")
-        );
-
-        let s = "*schema.table";
-        let err = parse_identifiers(s).expect_err("didn't fail to parse");
-        assert_eq!(
-            "SQL(ParserError(\"Unexpected token in identifier: *\"))",
-            format!("{err:?}")
-        );
-
-        let s = "schema.table*";
-        let err = parse_identifiers(s).expect_err("didn't fail to parse");
-        assert_eq!(
-            "SQL(ParserError(\"Unexpected token in identifier: *\"))",
-            format!("{err:?}")
-        );
-
-        let s = "schema.table.";
-        let err = parse_identifiers(s).expect_err("didn't fail to parse");
-        assert_eq!(
-            "SQL(ParserError(\"Trailing period in identifier\"))",
-            format!("{err:?}")
-        );
-
-        let s = "schema.*";
-        let err = parse_identifiers(s).expect_err("didn't fail to parse");
-        assert_eq!(
-            "SQL(ParserError(\"Unexpected token following period in identifier: *\"))",
-            format!("{err:?}")
-        );
-
-        Ok(())
-    }
-
     #[test]
     fn test_quote_identifier() -> Result<()> {
         let cases = vec![
@@ -696,4 +620,24 @@ mod tests {
         assert_eq!(longest_consecutive_prefix([0, 1, 2, 3, 4]), 5);
         assert_eq!(longest_consecutive_prefix([1, 2, 3, 4]), 0);
     }
+
+    #[test]
+    fn arc_data_ptr_eq() {
+        let x = Arc::new(());
+        let y = Arc::new(());
+        let y_clone = Arc::clone(&y);
+
+        assert!(
+            Arc::data_ptr_eq(&x, &x),
+            "same `Arc`s should point to same data"
+        );
+        assert!(
+            !Arc::data_ptr_eq(&x, &y),
+            "different `Arc`s should point to different data"
+        );
+        assert!(
+            Arc::data_ptr_eq(&y, &y_clone),
+            "cloned `Arc` should point to same data as the original"
+        );
+    }
 }
