@@ -49,9 +49,9 @@ use std::sync::Arc;
 use std::task::Poll;
 
 use crate::error::Result;
-use crate::execution::context::TaskContext;
-use crate::execution::memory_pool::MemoryConsumer;
 use crate::physical_plan::coalesce_batches::concat_batches;
+use datafusion_execution::memory_pool::MemoryConsumer;
+use datafusion_execution::TaskContext;
 
 /// Data of the inner table side
 type JoinLeftData = (RecordBatch, MemoryReservation);
