@@ -47,14 +47,14 @@ mod window_agg_exec;
 pub use bounded_window_agg_exec::BoundedWindowAggExec;
 pub use bounded_window_agg_exec::PartitionSearchMode;
 use datafusion_common::utils::longest_consecutive_prefix;
+use datafusion_physical_expr::equivalence::OrderingEquivalenceBuilder;
 use datafusion_physical_expr::expressions::Column;
 use datafusion_physical_expr::utils::{convert_to_expr, get_indices_of_matching_exprs};
 pub use datafusion_physical_expr::window::{
     BuiltInWindowExpr, PlainAggregateWindowExpr, WindowExpr,
 };
 use datafusion_physical_expr::{
-    normalize_expr_with_equivalence_properties, OrderedColumn,
-    OrderingEquivalenceProperties, PhysicalSortRequirement,
+    OrderedColumn, OrderingEquivalenceProperties, PhysicalSortRequirement,
 };
 pub use window_agg_exec::WindowAggExec;
 
@@ -254,15 +254,10 @@ pub(crate) fn window_ordering_equivalence(
 ) -> OrderingEquivalenceProperties {
     // We need to update the schema, so we can not directly use
     // `input.ordering_equivalence_properties()`.
-    let mut result = OrderingEquivalenceProperties::new(schema.clone());
-    result.extend(
-        input
-            .ordering_equivalence_properties()
-            .classes()
-            .iter()
-            .cloned(),
-    );
-    let out_ordering = input.output_ordering().unwrap_or(&[]);
+    let mut builder = OrderingEquivalenceBuilder::new(schema.clone())
+        .with_equivalences(input.equivalence_properties())
+        .with_existing_ordering(input.output_ordering().map(|elem| elem.to_vec()))
+        .extend(input.ordering_equivalence_properties());
     for expr in window_expr {
         if let Some(builtin_window_expr) =
             expr.as_any().downcast_ref::<BuiltInWindowExpr>()
@@ -274,35 +269,21 @@ pub(crate) fn window_ordering_equivalence(
                 .as_any()
                 .is::<RowNumber>()
             {
-                // If there is an existing ordering, add new ordering as an equivalence:
-                if let Some(first) = out_ordering.first() {
-                    // Normalize expression, as we search for ordering equivalences
-                    // on normalized versions:
-                    let normalized = normalize_expr_with_equivalence_properties(
-                        first.expr.clone(),
-                        input.equivalence_properties().classes(),
-                    );
-                    if let Some(column) = normalized.as_any().downcast_ref::<Column>() {
-                        let column_info =
-                            schema.column_with_name(expr.field().unwrap().name());
-                        if let Some((idx, field)) = column_info {
-                            let lhs = OrderedColumn::new(column.clone(), first.options);
-                            let options = SortOptions {
-                                descending: false,
-                                nulls_first: false,
-                            }; // ASC, NULLS LAST
-                            let rhs = OrderedColumn::new(
-                                Column::new(field.name(), idx),
-                                options,
-                            );
-                            result.add_equal_conditions((&lhs, &rhs));
-                        }
-                    }
+                if let Some((idx, field)) =
+                    schema.column_with_name(expr.field().unwrap().name())
+                {
+                    let column = Column::new(field.name(), idx);
+                    let options = SortOptions {
+                        descending: false,
+                        nulls_first: false,
+                    }; // ASC, NULLS LAST
+                    let rhs = OrderedColumn::new(column, options);
+                    builder.add_equal_conditions(vec![rhs]);
                 }
             }
         }
     }
-    result
+    builder.build()
 }
 #[cfg(test)]
 mod tests {
