@@ -99,14 +99,14 @@ async fn test_fn_bit_length() -> Result<()> {
     let expr = bit_length(col("a"));
 
     let expected = vec![
-        "+-------------------+",
-        "| bitlength(test.a) |",
-        "+-------------------+",
-        "| 48                |",
-        "| 48                |",
-        "| 48                |",
-        "| 72                |",
-        "+-------------------+",
+        "+--------------------+",
+        "| bit_length(test.a) |",
+        "+--------------------+",
+        "| 48                 |",
+        "| 48                 |",
+        "| 48                 |",
+        "| 72                 |",
+        "+--------------------+",
     ];
     assert_fn_batches!(expr, expected);
 
@@ -155,11 +155,11 @@ async fn test_fn_approx_median() -> Result<()> {
     let expr = approx_median(col("b"));
 
     let expected = vec![
-        "+----------------------+",
-        "| APPROXMEDIAN(test.b) |",
-        "+----------------------+",
-        "| 10                   |",
-        "+----------------------+",
+        "+-----------------------+",
+        "| APPROX_MEDIAN(test.b) |",
+        "+-----------------------+",
+        "| 10                    |",
+        "+-----------------------+",
     ];
 
     let df = create_test_table().await?;
@@ -175,11 +175,11 @@ async fn test_fn_approx_percentile_cont() -> Result<()> {
     let expr = approx_percentile_cont(col("b"), lit(0.5));
 
     let expected = vec![
-        "+-------------------------------------------+",
-        "| APPROXPERCENTILECONT(test.b,Float64(0.5)) |",
-        "+-------------------------------------------+",
-        "| 10                                        |",
-        "+-------------------------------------------+",
+        "+---------------------------------------------+",
+        "| APPROX_PERCENTILE_CONT(test.b,Float64(0.5)) |",
+        "+---------------------------------------------+",
+        "| 10                                          |",
+        "+---------------------------------------------+",
     ];
 
     let df = create_test_table().await?;
@@ -196,14 +196,14 @@ async fn test_fn_character_length() -> Result<()> {
     let expr = character_length(col("a"));
 
     let expected = vec![
-        "+-------------------------+",
-        "| characterlength(test.a) |",
-        "+-------------------------+",
-        "| 6                       |",
-        "| 6                       |",
-        "| 6                       |",
-        "| 9                       |",
-        "+-------------------------+",
+        "+--------------------------+",
+        "| character_length(test.a) |",
+        "+--------------------------+",
+        "| 6                        |",
+        "| 6                        |",
+        "| 6                        |",
+        "| 9                        |",
+        "+--------------------------+",
     ];
 
     assert_fn_batches!(expr, expected);
@@ -395,14 +395,14 @@ async fn test_fn_regexp_match() -> Result<()> {
     let expr = regexp_match(vec![col("a"), lit("[a-z]")]);
 
     let expected = vec![
-        "+-----------------------------------+",
-        "| regexpmatch(test.a,Utf8(\"[a-z]\")) |",
-        "+-----------------------------------+",
-        "| [a]                               |",
-        "| [a]                               |",
-        "| [d]                               |",
-        "| [b]                               |",
-        "+-----------------------------------+",
+        "+------------------------------------+",
+        "| regexp_match(test.a,Utf8(\"[a-z]\")) |",
+        "+------------------------------------+",
+        "| [a]                                |",
+        "| [a]                                |",
+        "| [d]                                |",
+        "| [b]                                |",
+        "+------------------------------------+",
     ];
 
     assert_fn_batches!(expr, expected);
@@ -416,14 +416,14 @@ async fn test_fn_regexp_replace() -> Result<()> {
     let expr = regexp_replace(vec![col("a"), lit("[a-z]"), lit("x"), lit("g")]);
 
     let expected = vec![
-        "+---------------------------------------------------------+",
-        "| regexpreplace(test.a,Utf8(\"[a-z]\"),Utf8(\"x\"),Utf8(\"g\")) |",
-        "+---------------------------------------------------------+",
-        "| xxxDEF                                                  |",
-        "| xxx123                                                  |",
-        "| CBAxxx                                                  |",
-        "| 123AxxDxx                                               |",
-        "+---------------------------------------------------------+",
+        "+----------------------------------------------------------+",
+        "| regexp_replace(test.a,Utf8(\"[a-z]\"),Utf8(\"x\"),Utf8(\"g\")) |",
+        "+----------------------------------------------------------+",
+        "| xxxDEF                                                   |",
+        "| xxx123                                                   |",
+        "| CBAxxx                                                   |",
+        "| 123AxxDxx                                                |",
+        "+----------------------------------------------------------+",
     ];
 
     assert_fn_batches!(expr, expected);
@@ -581,14 +581,14 @@ async fn test_fn_split_part() -> Result<()> {
     let expr = split_part(col("a"), lit("b"), lit(1));
 
     let expected = vec![
-        "+--------------------------------------+",
-        "| splitpart(test.a,Utf8(\"b\"),Int32(1)) |",
-        "+--------------------------------------+",
-        "| a                                    |",
-        "| a                                    |",
-        "| CBAdef                               |",
-        "| 123A                                 |",
-        "+--------------------------------------+",
+        "+---------------------------------------+",
+        "| split_part(test.a,Utf8(\"b\"),Int32(1)) |",
+        "+---------------------------------------+",
+        "| a                                     |",
+        "| a                                     |",
+        "| CBAdef                                |",
+        "| 123A                                  |",
+        "+---------------------------------------+",
     ];
     assert_fn_batches!(expr, expected);
 
@@ -600,14 +600,14 @@ async fn test_fn_starts_with() -> Result<()> {
     let expr = starts_with(col("a"), lit("abc"));
 
     let expected = vec![
-        "+--------------------------------+",
-        "| startswith(test.a,Utf8(\"abc\")) |",
-        "+--------------------------------+",
-        "| true                           |",
-        "| true                           |",
-        "| false                          |",
-        "| false                          |",
-        "+--------------------------------+",
+        "+---------------------------------+",
+        "| starts_with(test.a,Utf8(\"abc\")) |",
+        "+---------------------------------+",
+        "| true                            |",
+        "| true                            |",
+        "| false                           |",
+        "| false                           |",
+        "+---------------------------------+",
     ];
 
     assert_fn_batches!(expr, expected);
@@ -679,14 +679,14 @@ async fn test_fn_to_hex() -> Result<()> {
     let expr = to_hex(col("b"));
 
     let expected = vec![
-        "+---------------+",
-        "| tohex(test.b) |",
-        "+---------------+",
-        "| 1             |",
-        "| a             |",
-        "| a             |",
-        "| 64            |",
-        "+---------------+",
+        "+----------------+",
+        "| to_hex(test.b) |",
+        "+----------------+",
+        "| 1              |",
+        "| a              |",
+        "| a              |",
+        "| 64             |",
+        "+----------------+",
     ];
     assert_fn_batches!(expr, expected);
 
