@@ -29,7 +29,7 @@ async fn csv_query_array_agg_distinct() -> Result<()> {
 
     // The results for this query should be something like the following:
     //    +------------------------------------------+
-    //    | ARRAYAGG(DISTINCT aggregate_test_100.c2) |
+    //    | ARRAY_AGG(DISTINCT aggregate_test_100.c2) |
     //    +------------------------------------------+
     //    | [4, 2, 3, 5, 1]                          |
     //    +------------------------------------------+
@@ -37,7 +37,7 @@ async fn csv_query_array_agg_distinct() -> Result<()> {
     assert_eq!(
         *actual[0].schema(),
         Schema::new(vec![Field::new_list(
-            "ARRAYAGG(DISTINCT aggregate_test_100.c2)",
+            "ARRAY_AGG(DISTINCT aggregate_test_100.c2)",
             Field::new("item", DataType::UInt32, true),
             false
         ),])
