@@ -32,7 +32,6 @@ use std::collections::HashSet;
 use std::fmt;
 use std::fmt::{Display, Formatter, Write};
 use std::hash::{BuildHasher, Hash, Hasher};
-use std::ops::Not;
 use std::sync::Arc;
 
 /// `Expr` is a central struct of DataFusion's query API, and
@@ -740,43 +739,6 @@ impl Expr {
         binary_expr(self, Operator::Or, other)
     }
 
-    /// Return `self & other`
-    pub fn bitwise_and(self, other: Expr) -> Expr {
-        binary_expr(self, Operator::BitwiseAnd, other)
-    }
-
-    /// Return `self | other`
-    pub fn bitwise_or(self, other: Expr) -> Expr {
-        binary_expr(self, Operator::BitwiseOr, other)
-    }
-
-    /// Return `self ^ other`
-    pub fn bitwise_xor(self, other: Expr) -> Expr {
-        binary_expr(self, Operator::BitwiseXor, other)
-    }
-
-    /// Return `self >> other`
-    pub fn bitwise_shift_right(self, other: Expr) -> Expr {
-        binary_expr(self, Operator::BitwiseShiftRight, other)
-    }
-
-    /// Return `self << other`
-    pub fn bitwise_shift_left(self, other: Expr) -> Expr {
-        binary_expr(self, Operator::BitwiseShiftLeft, other)
-    }
-
-    /// Return `!self`
-    #[allow(clippy::should_implement_trait)]
-    pub fn not(self) -> Expr {
-        !self
-    }
-
-    /// Calculate the modulus of two expressions.
-    /// Return `self % other`
-    pub fn modulus(self, other: Expr) -> Expr {
-        binary_expr(self, Operator::Modulo, other)
-    }
-
     /// Return `self LIKE other`
     pub fn like(self, other: Expr) -> Expr {
         Expr::Like(Like::new(false, Box::new(self), Box::new(other), None))
@@ -817,13 +779,11 @@ impl Expr {
     }
 
     /// Return `IsNull(Box(self))
-    #[allow(clippy::wrong_self_convention)]
     pub fn is_null(self) -> Expr {
         Expr::IsNull(Box::new(self))
     }
 
     /// Return `IsNotNull(Box(self))
-    #[allow(clippy::wrong_self_convention)]
     pub fn is_not_null(self) -> Expr {
         Expr::IsNotNull(Box::new(self))
     }
@@ -909,34 +869,6 @@ impl Expr {
     }
 }
 
-impl Not for Expr {
-    type Output = Self;
-
-    fn not(self) -> Self::Output {
-        match self {
-            Expr::Like(Like {
-                negated,
-                expr,
-                pattern,
-                escape_char,
-            }) => Expr::Like(Like::new(!negated, expr, pattern, escape_char)),
-            Expr::ILike(Like {
-                negated,
-                expr,
-                pattern,
-                escape_char,
-            }) => Expr::ILike(Like::new(!negated, expr, pattern, escape_char)),
-            Expr::SimilarTo(Like {
-                negated,
-                expr,
-                pattern,
-                escape_char,
-            }) => Expr::SimilarTo(Like::new(!negated, expr, pattern, escape_char)),
-            _ => Expr::Not(Box::new(self)),
-        }
-    }
-}
-
 /// Format expressions for display as part of a logical plan. In many cases, this will produce
 /// similar output to `Expr.name()` except that column names will be prefixed with '#'.
 impl fmt::Display for Expr {
@@ -1548,11 +1480,6 @@ mod test {
         Ok(())
     }
 
-    #[test]
-    fn test_not() {
-        assert_eq!(lit(1).not(), !lit(1));
-    }
-
     #[test]
     fn test_partial_ord() {
         // Test validates that partial ord is defined for Expr using hashes, not
