@@ -18,8 +18,9 @@
 //! Rewrite for order by expressions
 
 use crate::expr::Sort;
-use crate::expr_rewriter::{normalize_col, rewrite_expr};
+use crate::expr_rewriter::normalize_col;
 use crate::{Cast, Expr, ExprSchemable, LogicalPlan, TryCast};
+use datafusion_common::tree_node::{Transformed, TreeNode};
 use datafusion_common::{Column, Result};
 
 /// Rewrite sort on aggregate expressions to sort on the column of aggregate output
@@ -82,7 +83,7 @@ fn rewrite_in_terms_of_projection(
 ) -> Result<Expr> {
     // assumption is that each item in exprs, such as "b + c" is
     // available as an output column named "b + c"
-    rewrite_expr(expr, |expr| {
+    expr.transform(&|expr| {
         // search for unnormalized names first such as "c1" (such as aliases)
         if let Some(found) = proj_exprs.iter().find(|a| (**a) == expr) {
             let col = Expr::Column(
@@ -90,7 +91,7 @@ fn rewrite_in_terms_of_projection(
                     .to_field(input.schema())
                     .map(|f| f.qualified_column())?,
             );
-            return Ok(col);
+            return Ok(Transformed::Yes(col));
         }
 
         // if that doesn't work, try to match the expression as an
@@ -102,7 +103,7 @@ fn rewrite_in_terms_of_projection(
             e
         } else {
             // The expr is not based on Aggregate plan output. Skip it.
-            return Ok(expr);
+            return Ok(Transformed::No(expr));
         };
 
         // expr is an actual expr like min(t.c2), but we are looking
@@ -117,7 +118,7 @@ fn rewrite_in_terms_of_projection(
         // look for the column named the same as this expr
         if let Some(found) = proj_exprs.iter().find(|a| expr_match(&search_col, a)) {
             let found = found.clone();
-            let expr = match normalized_expr {
+            return Ok(Transformed::Yes(match normalized_expr {
                 Expr::Cast(Cast { expr: _, data_type }) => Expr::Cast(Cast {
                     expr: Box::new(found),
                     data_type,
@@ -127,10 +128,10 @@ fn rewrite_in_terms_of_projection(
                     data_type,
                 }),
                 _ => found,
-            };
-            return Ok(expr);
+            }));
         }
-        Ok(expr)
+
+        Ok(Transformed::No(expr))
     })
 }
 
