@@ -1133,16 +1133,18 @@ pub fn project_with_column_index(
         .into_iter()
         .enumerate()
         .map(|(i, e)| match e {
-            alias @ Expr::Alias { .. }
-                if &alias.display_name().unwrap() != schema.field(i).name() =>
-            {
-                alias.unalias().alias(schema.field(i).name())
+            Expr::Alias(_, ref name) if name != schema.field(i).name() => {
+                e.unalias().alias(schema.field(i).name())
             }
-            ignore_alias @ Expr::Alias { .. } => ignore_alias,
-            ignore_col @ Expr::Column { .. } => ignore_col,
-            expr => expr.alias(schema.field(i).name()),
+            Expr::Column(Column {
+                relation: _,
+                ref name,
+            }) if name != schema.field(i).name() => e.alias(schema.field(i).name()),
+            Expr::Alias { .. } | Expr::Column { .. } => e,
+            _ => e.alias(schema.field(i).name()),
         })
         .collect::<Vec<_>>();
+
     Ok(LogicalPlan::Projection(Projection::try_new_with_schema(
         alias_expr, input, schema,
     )?))
@@ -1234,11 +1236,10 @@ pub fn project(
         let e = e.into();
         match e {
             Expr::Wildcard => {
-                projected_expr.extend(expand_wildcard(input_schema, &plan)?)
-            }
-            Expr::QualifiedWildcard { ref qualifier } => {
-                projected_expr.extend(expand_qualified_wildcard(qualifier, input_schema)?)
+                projected_expr.extend(expand_wildcard(input_schema, &plan, None)?)
             }
+            Expr::QualifiedWildcard { ref qualifier } => projected_expr
+                .extend(expand_qualified_wildcard(qualifier, input_schema, None)?),
             _ => projected_expr
                 .push(columnize_expr(normalize_col(e, &plan)?, input_schema)),
         }
@@ -1326,7 +1327,7 @@ pub fn wrap_projection_for_join_if_necessary(
 
     let need_project = join_keys.iter().any(|key| !matches!(key, Expr::Column(_)));
     let plan = if need_project {
-        let mut projection = expand_wildcard(input_schema, &input)?;
+        let mut projection = expand_wildcard(input_schema, &input, None)?;
         let join_key_items = alias_join_keys
             .iter()
             .flat_map(|expr| expr.try_into_col().is_err().then_some(expr))
