@@ -203,14 +203,7 @@ pub fn coerce_types(
     };
 
     // re-write the error message of failed coercions to include the operator's information
-    match result {
-        None => Err(DataFusionError::Plan(
-            format!(
-                "{lhs_type:?} {op} {rhs_type:?} can't be evaluated because there isn't a common type to coerce the types to"
-            ),
-        )),
-        Some(t) => Ok(t)
-    }
+    result.ok_or(DataFusionError::Plan(format!("{lhs_type:?} {op} {rhs_type:?} can't be evaluated because there isn't a common type to coerce the types to")))
 }
 
 /// Coercion rules for mathematics operators between decimal and non-decimal types.
