@@ -19,9 +19,10 @@ use crate::planner::{ContextProvider, PlannerContext, SqlToRel};
 use arrow::compute::kernels::cast_utils::parse_interval_month_day_nano;
 use arrow_schema::DataType;
 use datafusion_common::{DFSchema, DataFusionError, Result, ScalarValue};
-use datafusion_expr::{lit, Expr};
+use datafusion_expr::expr::{BinaryExpr, Placeholder};
+use datafusion_expr::{lit, Expr, Operator};
 use log::debug;
-use sqlparser::ast::{DateTimeField, Expr as SQLExpr, Value};
+use sqlparser::ast::{BinaryOperator, DateTimeField, Expr as SQLExpr, Value};
 use sqlparser::parser::ParserError::ParserError;
 use std::collections::HashSet;
 
@@ -47,14 +48,9 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
 
     /// Parse number in sql string, convert to Expr::Literal
     fn parse_sql_number(&self, n: &str) -> Result<Expr> {
-        if n.find('E').is_some() {
-            // not implemented yet
-            // https://github.com/apache/arrow-datafusion/issues/3448
-            Err(DataFusionError::NotImplemented(
-                "sql numeric literals in scientific notation are not supported"
-                    .to_string(),
-            ))
-        } else if let Ok(n) = n.parse::<i64>() {
+        if let Ok(n) = n.parse::<i64>() {
+            Ok(lit(n))
+        } else if let Ok(n) = n.parse::<u64>() {
             Ok(lit(n))
         } else if self.options.parse_float_as_decimal {
             // remove leading zeroes
@@ -118,10 +114,10 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             param, param_type
         );
 
-        Ok(Expr::Placeholder {
-            id: param,
-            data_type: param_type.cloned(),
-        })
+        Ok(Expr::Placeholder(Placeholder::new(
+            param,
+            param_type.cloned(),
+        )))
     }
 
     pub(super) fn sql_array_literal(
@@ -165,9 +161,17 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         }
     }
 
+    /// Convert a SQL interval expression to a DataFusion logical plan
+    /// expression
+    ///
+    /// Waiting for this issue to be resolved:
+    /// `<https://github.com/sqlparser-rs/sqlparser-rs/issues/869>`
+    #[allow(clippy::too_many_arguments)]
     pub(super) fn sql_interval_to_expr(
         &self,
         value: SQLExpr,
+        schema: &DFSchema,
+        planner_context: &mut PlannerContext,
         leading_field: Option<DateTimeField>,
         leading_precision: Option<u64>,
         last_field: Option<DateTimeField>,
@@ -196,6 +200,92 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             SQLExpr::Value(
                 Value::SingleQuotedString(s) | Value::DoubleQuotedString(s),
             ) => s,
+            // Support expressions like `interval '1 month' + date/timestamp`.
+            // Such expressions are parsed like this by sqlparser-rs
+            //
+            // Interval
+            //   BinaryOp
+            //     Value(StringLiteral)
+            //     Cast
+            //       Value(StringLiteral)
+            //
+            // This code rewrites them to the following:
+            //
+            // BinaryOp
+            //   Interval
+            //     Value(StringLiteral)
+            //   Cast
+            //      Value(StringLiteral)
+            SQLExpr::BinaryOp { left, op, right } => {
+                let df_op = match op {
+                    BinaryOperator::Plus => Operator::Plus,
+                    BinaryOperator::Minus => Operator::Minus,
+                    _ => {
+                        return Err(DataFusionError::NotImplemented(format!(
+                            "Unsupported interval operator: {op:?}"
+                        )));
+                    }
+                };
+                match (leading_field, left.as_ref(), right.as_ref()) {
+                    (_, _, SQLExpr::Value(_)) => {
+                        let left_expr = self.sql_interval_to_expr(
+                            *left,
+                            schema,
+                            planner_context,
+                            leading_field,
+                            None,
+                            None,
+                            None,
+                        )?;
+                        let right_expr = self.sql_interval_to_expr(
+                            *right,
+                            schema,
+                            planner_context,
+                            leading_field,
+                            None,
+                            None,
+                            None,
+                        )?;
+                        return Ok(Expr::BinaryExpr(BinaryExpr::new(
+                            Box::new(left_expr),
+                            df_op,
+                            Box::new(right_expr),
+                        )));
+                    }
+                    // In this case, the left node is part of the interval
+                    // expr and the right node is an independent expr.
+                    //
+                    // Leading field is not supported when the right operand
+                    // is not a value.
+                    (None, _, _) => {
+                        let left_expr = self.sql_interval_to_expr(
+                            *left,
+                            schema,
+                            planner_context,
+                            None,
+                            None,
+                            None,
+                            None,
+                        )?;
+                        let right_expr = self.sql_expr_to_logical_expr(
+                            *right,
+                            schema,
+                            planner_context,
+                        )?;
+                        return Ok(Expr::BinaryExpr(BinaryExpr::new(
+                            Box::new(left_expr),
+                            df_op,
+                            Box::new(right_expr),
+                        )));
+                    }
+                    _ => {
+                        let value = SQLExpr::BinaryOp { left, op, right };
+                        return Err(DataFusionError::NotImplemented(format!(
+                            "Unsupported interval argument. Expected string literal, got: {value:?}"
+                        )));
+                    }
+                }
+            }
             _ => {
                 return Err(DataFusionError::NotImplemented(format!(
                     "Unsupported interval argument. Expected string literal, got: {value:?}"
