@@ -16,7 +16,7 @@
 // under the License.
 
 use crate::parser::{
-    CopyToStatement, CreateExternalTable, DFParser, DescribeTableStmt,
+    CopyToStatement, CreateExternalTable, DFParser, DescribeTableStmt, LexOrdering,
     Statement as DFStatement,
 };
 use crate::planner::{
@@ -44,9 +44,9 @@ use datafusion_expr::{
 };
 use sqlparser::ast;
 use sqlparser::ast::{
-    Assignment, Expr as SQLExpr, Expr, Ident, ObjectName, ObjectType, OrderByExpr, Query,
-    SchemaName, SetExpr, ShowCreateObject, ShowStatementFilter, Statement,
-    TableConstraint, TableFactor, TableWithJoins, TransactionMode, UnaryOperator, Value,
+    Assignment, Expr as SQLExpr, Expr, Ident, ObjectName, ObjectType, Query, SchemaName,
+    SetExpr, ShowCreateObject, ShowStatementFilter, Statement, TableConstraint,
+    TableFactor, TableWithJoins, TransactionMode, UnaryOperator, Value,
 };
 
 use datafusion_expr::expr::Placeholder;
@@ -404,11 +404,18 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
             }
 
             Statement::Delete {
-                table_name,
+                tables,
                 using,
                 selection,
                 returning,
+                from,
             } => {
+                if !tables.is_empty() {
+                    return Err(DataFusionError::NotImplemented(
+                        "DELETE <TABLE> not supported".to_string(),
+                    ));
+                }
+
                 if using.is_some() {
                     Err(DataFusionError::Plan(
                         "Using clause not supported".to_owned(),
@@ -419,6 +426,7 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                         "Delete-returning clause not yet supported".to_owned(),
                     ))?;
                 }
+                let table_name = self.get_delete_target(from)?;
                 self.delete_to_plan(table_name, selection)
             }
 
@@ -493,6 +501,28 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
         }
     }
 
+    fn get_delete_target(&self, mut from: Vec<TableWithJoins>) -> Result<ObjectName> {
+        if from.len() != 1 {
+            return Err(DataFusionError::NotImplemented(format!(
+                "DELETE FROM only supports single table, got {}: {from:?}",
+                from.len()
+            )));
+        }
+        let table_factor = from.pop().unwrap();
+        if !table_factor.joins.is_empty() {
+            return Err(DataFusionError::NotImplemented(
+                "DELETE FROM only supports single table, got: joins".to_string(),
+            ));
+        }
+        let TableFactor::Table{name, ..} = table_factor.relation else {
+            return Err(DataFusionError::NotImplemented(format!(
+                "DELETE FROM only supports single table, got: {table_factor:?}"
+            )))
+        };
+
+        Ok(name)
+    }
+
     /// Generate a logical plan from a "SHOW TABLES" query
     fn show_tables_to_plan(
         &self,
@@ -548,10 +578,10 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
 
     fn build_order_by(
         &self,
-        order_exprs: Vec<OrderByExpr>,
+        order_exprs: Vec<LexOrdering>,
         schema: &DFSchemaRef,
         planner_context: &mut PlannerContext,
-    ) -> Result<Vec<datafusion_expr::Expr>> {
+    ) -> Result<Vec<Vec<datafusion_expr::Expr>>> {
         // Ask user to provide a schema if schema is empty.
         if !order_exprs.is_empty() && schema.fields().is_empty() {
             return Err(DataFusionError::Plan(
@@ -559,25 +589,26 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
                     .to_owned(),
             ));
         }
-        // Convert each OrderByExpr to a SortExpr:
-        let result = order_exprs
-            .into_iter()
-            .map(|e| self.order_by_to_sort_expr(e, schema, planner_context))
-            .collect::<Result<Vec<_>>>()?;
-        // Verify that columns of all SortExprs exist in the schema:
-        for expr in result.iter() {
-            for column in expr.to_columns()?.iter() {
-                if !schema.has_column(column) {
-                    // Return an error if any column is not in the schema:
-                    return Err(DataFusionError::Plan(format!(
-                        "Column {column} is not in schema"
-                    )));
+
+        let mut all_results = vec![];
+        for expr in order_exprs {
+            // Convert each OrderByExpr to a SortExpr:
+            let expr_vec = self.order_by_to_sort_expr(&expr, schema, planner_context)?;
+            // Verify that columns of all SortExprs exist in the schema:
+            for expr in expr_vec.iter() {
+                for column in expr.to_columns()?.iter() {
+                    if !schema.has_column(column) {
+                        // Return an error if any column is not in the schema:
+                        return Err(DataFusionError::Plan(format!(
+                            "Column {column} is not in schema"
+                        )));
+                    }
                 }
             }
+            // If all SortExprs are valid, return them as an expression vector
+            all_results.push(expr_vec)
         }
-
-        // If all SortExprs are valid, return them as an expression vector
-        Ok(result)
+        Ok(all_results)
     }
 
     /// Generate a logical plan from a CREATE EXTERNAL TABLE statement
@@ -789,16 +820,9 @@ impl<'a, S: ContextProvider> SqlToRel<'a, S> {
 
     fn delete_to_plan(
         &self,
-        table_factor: TableFactor,
+        table_name: ObjectName,
         predicate_expr: Option<Expr>,
     ) -> Result<LogicalPlan> {
-        let table_name = match &table_factor {
-            TableFactor::Table { name, .. } => name.clone(),
-            _ => Err(DataFusionError::Plan(
-                "Cannot delete from non-table relations!".to_string(),
-            ))?,
-        };
-
         // Do a table lookup to verify the table exists
         let table_ref = self.object_name_to_table_reference(table_name.clone())?;
         let provider = self.schema_provider.get_table_provider(table_ref.clone())?;
